/**
 * DevExtreme (core/dom_component_with_template.js)
 * Version: 20.1.0 (build 19347-0311)
 * Build date: Fri Dec 13 2019
 *
 * Copyright (c) 2012 - 2019 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";
var _extend = require("./utils/extend");
var _dom_component = require("./dom_component");
var _dom_component2 = _interopRequireDefault(_dom_component);
var _template_manager = require("./template_manager");
var _template_manager2 = _interopRequireDefault(_template_manager);

function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        "default": obj
    }
}
var DOMComponentWithTemplate = _dom_component2.default.inherit({
    _getDefaultOptions: function() {
        return (0, _extend.extend)(this.callBase(), _template_manager2.default.createDefaultOptions())
    },
    _getAnonymousTemplateName: function() {
        return
    },
    _init: function() {
        this.callBase();
        var _option = this.option(),
            _option$integrationOp = _option.integrationOptions,
            integrationOptions = void 0 === _option$integrationOp ? {} : _option$integrationOp;
        var createTemplate = integrationOptions.createTemplate;
        this._templateManager = new _template_manager2.default(createTemplate, this._getAnonymousTemplateName());
        this._initTemplates()
    },
    _dispose: function() {
        this._templateManager.dispose();
        this.callBase()
    },
    _initTemplates: function() {
        var _this = this;
        var _templateManager$extr = this._templateManager.extractTemplates(this.$element()),
            templates = _templateManager$extr.templates,
            anonymousTemplateMeta = _templateManager$extr.anonymousTemplateMeta;
        var anonymousTemplate = this.option("integrationOptions.templates." + anonymousTemplateMeta.name);
        templates.forEach(function(_ref) {
            var name = _ref.name,
                template = _ref.template;
            _this._setOptionSilent("integrationOptions.templates." + name, template)
        });
        if (anonymousTemplateMeta.name && !anonymousTemplate) {
            this._setOptionSilent("integrationOptions.templates." + anonymousTemplateMeta.name, anonymousTemplateMeta.template)
        }
    },
    _getTemplateByOption: function(optionName) {
        return this._getTemplate(this.option(optionName))
    },
    _getTemplate: function(templateSource) {
        var templates = this.option("integrationOptions.templates");
        var isAsyncTemplate = this.option("templatesRenderAsynchronously");
        var skipTemplates = this.option("integrationOptions.skipTemplates");
        return this._templateManager.getTemplate(templateSource, templates, {
            isAsyncTemplate: isAsyncTemplate,
            skipTemplates: skipTemplates
        }, this)
    },
    _saveTemplate: function(name, template) {
        this._setOptionSilent("integrationOptions.templates." + name, this._templateManager._createTemplate(template))
    }
});
module.exports = DOMComponentWithTemplate;
